@@ -18,8 +18,6 @@
 
 
 
-static GdkPixbuf * get_missing (GtkIconTheme *icon_theme,
-				int           preffered_size);
 static void button_widget_icon_theme_changed (ButtonWidget *button);
 static void button_widget_reload_pixbuf (ButtonWidget *button);
 
@@ -34,7 +32,7 @@ enum {
 
 #define BUTTON_WIDGET_DISPLACEMENT 2
 
-static GObjectClass *parent_class;
+G_DEFINE_TYPE (ButtonWidget, button_widget, GTK_TYPE_BUTTON);
 
 /* colorshift a pixbuf */
 static void
@@ -147,13 +145,11 @@ button_widget_unrealize (GtkWidget *widget)
 {
 	GtkButton *button;
 	PanelWidget *panel;
-	GtkWidget *parent;
 
 	g_return_if_fail (widget != NULL);
 	g_return_if_fail (BUTTON_IS_WIDGET (widget));
 
 	panel  = PANEL_WIDGET (widget->parent);
-	parent = GTK_WIDGET (panel->toplevel);
 	button = GTK_BUTTON (widget);
 
 	if (button->event_window != NULL) {
@@ -162,7 +158,7 @@ button_widget_unrealize (GtkWidget *widget)
 		button->event_window = NULL;
 	}
 
-	GTK_WIDGET_CLASS (parent_class)->unrealize (widget);
+	GTK_WIDGET_CLASS (button_widget_parent_class)->unrealize (widget);
 }
 
 static void
@@ -191,25 +187,17 @@ button_widget_reload_pixbuf (ButtonWidget *button)
 		button->pixbuf = panel_load_icon (button->icon_theme,
 						  button->filename,
 						  button->size,
+						  button->orientation & PANEL_VERTICAL_MASK   ? button->size : -1,
+						  button->orientation & PANEL_HORIZONTAL_MASK ? button->size : -1,
 						  &error);
 		if (error) {
-			panel_error_dialog (gdk_screen_get_default (),
-					    "cannot_load_pixbuf", TRUE,
-					    _("Failed to load image %s"),
-					    "%s",
-					    button->filename,
-					    error);
+			button->pixbuf = gtk_widget_render_icon (GTK_WIDGET (button),
+								 GTK_STOCK_MISSING_IMAGE,
+								 (GtkIconSize) -1, NULL);
 			g_free (error);
 		}
 	}
 
-	if (button->pixbuf == NULL)
-		button->pixbuf = get_missing (button->icon_theme,
-					      button->size);
-
-	if (button->pixbuf == NULL)
-		return;
-
 	button->pixbuf_hc = make_hc_pixbuf (button->pixbuf);
 
 	gtk_widget_queue_resize (GTK_WIDGET (button));
@@ -232,7 +220,7 @@ button_widget_finalize (GObject *object)
 	g_free (button->filename);
 	button->filename = NULL;
 
-	parent_class->finalize (object);
+	G_OBJECT_CLASS (button_widget_parent_class)->finalize (object);
 }
 
 static void
@@ -303,20 +291,6 @@ button_widget_set_property (GObject      *object,
 	}
 }
 
-static GdkPixbuf *
-get_missing (GtkIconTheme *icon_theme,
-	     int           preffered_size)
-{
-	GdkPixbuf *retval = NULL;
-
-	retval = gtk_icon_theme_load_icon (icon_theme, "gnome-unknown",
-					   preffered_size, 0, NULL);
-	if (retval == NULL)
-		retval = missing_pixbuf (preffered_size);
-
-	return retval;
-}
-
 static GtkArrowType
 calc_arrow (PanelOrientation  orientation,
 	    int               button_width,
@@ -506,7 +480,14 @@ button_widget_size_allocate (GtkWidget     *widget,
 	else
 		size = allocation->width;
 
-	size = CLAMP (size, 1, 48);
+	if (size < 22)
+		size = 16;
+	else if (size < 32)
+		size = 22;
+	else if (size < 48)
+		size = 32;
+	else
+		size = 48;
 
 	if (button_widget->size != size) {
 		button_widget->size = size;
@@ -517,10 +498,6 @@ button_widget_size_allocate (GtkWidget     *widget,
 	widget->allocation = *allocation;
 
 	if (GTK_WIDGET_REALIZED (widget)) {
-		PanelWidget *panel;
-
-		panel = PANEL_WIDGET (widget->parent);
-
 		gdk_window_move_resize (button->event_window, 
 					allocation->x,
 					allocation->y,
@@ -537,8 +514,8 @@ button_widget_activate (GtkButton *button)
 	if (!button_widget->activatable)
 		return;
 
-	if (GTK_BUTTON_CLASS (parent_class)->activate)
-		GTK_BUTTON_CLASS (parent_class)->activate (button);
+	if (GTK_BUTTON_CLASS (button_widget_parent_class)->activate)
+		GTK_BUTTON_CLASS (button_widget_parent_class)->activate (button);
 }
 
 static gboolean
@@ -547,8 +524,11 @@ button_widget_button_press (GtkWidget *widget, GdkEventButton *event)
 	g_return_val_if_fail (BUTTON_IS_WIDGET (widget), FALSE);
 	g_return_val_if_fail (event != NULL, FALSE);
 
-	if (event->button == 1 && BUTTON_WIDGET (widget)->activatable)
-		return GTK_WIDGET_CLASS (parent_class)->button_press_event (widget, event);
+	if (event->button == 1 && BUTTON_WIDGET (widget)->activatable &&
+	/* we don't want to have two/three "click" events for double/triple
+	 * clicks. FIXME: this is only a workaround, waiting for bug 159101 */
+	    event->type == GDK_BUTTON_PRESS)
+		return GTK_WIDGET_CLASS (button_widget_parent_class)->button_press_event (widget, event);
 
 	return FALSE;
 }
@@ -561,7 +541,7 @@ button_widget_enter_notify (GtkWidget *widget, GdkEventCrossing *event)
 	g_return_val_if_fail (BUTTON_IS_WIDGET (widget), FALSE);
 
 	in_button = GTK_BUTTON (widget)->in_button;
-	GTK_WIDGET_CLASS (parent_class)->enter_notify_event (widget, event);
+	GTK_WIDGET_CLASS (button_widget_parent_class)->enter_notify_event (widget, event);
 	if (in_button != GTK_BUTTON (widget)->in_button &&
 	    panel_global_config_get_highlight_when_over ())
 		gtk_widget_queue_draw (widget);
@@ -577,7 +557,7 @@ button_widget_leave_notify (GtkWidget *widget, GdkEventCrossing *event)
 	g_return_val_if_fail (BUTTON_IS_WIDGET (widget), FALSE);
 
 	in_button = GTK_BUTTON (widget)->in_button;
-	GTK_WIDGET_CLASS (parent_class)->leave_notify_event (widget, event);
+	GTK_WIDGET_CLASS (button_widget_parent_class)->leave_notify_event (widget, event);
 	if (in_button != GTK_BUTTON (widget)->in_button &&
 	    panel_global_config_get_highlight_when_over ())
 		gtk_widget_queue_draw (widget);
@@ -586,7 +566,7 @@ button_widget_leave_notify (GtkWidget *widget, GdkEventCrossing *event)
 }
 
 static void
-button_widget_instance_init (ButtonWidget *button)
+button_widget_init (ButtonWidget *button)
 {
 	button->icon_theme = NULL;
 	button->pixbuf     = NULL;
@@ -611,8 +591,6 @@ button_widget_class_init (ButtonWidgetClass *klass)
 	GtkWidgetClass *widget_class  = (GtkWidgetClass *) klass;
 	GtkButtonClass *button_class  = (GtkButtonClass *) klass;
 
-	parent_class = g_type_class_peek_parent (klass);
-
 	gobject_class->finalize     = button_widget_finalize;
 	gobject_class->get_property = button_widget_get_property;
 	gobject_class->set_property = button_widget_set_property;
@@ -675,31 +653,6 @@ button_widget_class_init (ButtonWidgetClass *klass)
 					     G_PARAM_READWRITE));
 }
 
-GType
-button_widget_get_type (void)
-{
-	static GType object_type = 0;
-
-	if (object_type == 0) {
-		static const GTypeInfo object_info = {
-			sizeof (ButtonWidgetClass),
-                    	(GBaseInitFunc)         NULL,
-                    	(GBaseFinalizeFunc)     NULL,
-                    	(GClassInitFunc)        button_widget_class_init,
-                    	NULL,                   /* class_finalize */
-                    	NULL,                   /* class_data */
-                    	sizeof (ButtonWidget),
-                    	0,                      /* n_preallocs */
-                    	(GInstanceInitFunc)     button_widget_instance_init
-
-		};
-
-		object_type = g_type_register_static (GTK_TYPE_BUTTON, "ButtonWidget", &object_info, 0);
-	}
-
-	return object_type;
-}
-
 GtkWidget *
 button_widget_new (const char       *filename,
 		   gboolean          arrow,
